'use strict';

function jsonLdNodeToName(node) {
  var _a;
  const nameThing = (_a = node["https://schema.org/name"]) !== null && _a !== void 0 ? _a : node["http://schema.org/name"];
  if (Array.isArray(nameThing) && nameThing[0] !== undefined && typeof nameThing[0] === "object") {
    let nameObject = nameThing[0];
    return nameObject["@value"];
  }
  else if (Array.isArray(nameThing) && nameThing[0] !== undefined && typeof nameThing[0] === "string") {
    return nameThing[0];
  }
  else if (typeof nameThing === "string") {
    return nameThing;
  }
  else if (node["@id"] !== undefined) {
    return node["@id"];
  }
  else {
    return "Unknown";
  }
}
/**
 * Extracts the types (their IDs) from a JSON-LD node object.
 * @param node The JSON-LD node object to extract type IDs from.
 * @returns An array of type IDs for the given node object.
 */
function jsonLdNodeToTypeIds(node) {
  if (node["@type"] !== undefined) {
    if (Array.isArray(node["@type"])) {
      return node["@type"];
    }
    else {
      return [node["@type"]];
    }
  }
  else {
    return [];
  }
}
/**
 * Extracts the types suffixes/names from the given JSON-LD node object.
 * **It does not return the full types but their suffixes/names.**
 * @param node - The JSON-LD node object to extract types from.
 * @returns An array of strings representing the extracted types.
 * @deprecated Use jsonLdNodeToTypeIds and jsonLDNodeToName instead, as it is bad practice to extract the name from the ID.
 */
function jsonLdNodeToType(node) {
  if (node["@type"] !== undefined) {
    if (Array.isArray(node["@type"])) {
      return node["@type"].map(x => x.replace("http://schema.org/", ""));
    }
    else {
      return [node["@type"].replace("http://schema.org/", "")];
    }
  }
  else {
    return [];
  }
}
/**
 * Returns the index of the first element in the given array of JSON-LD node objects that has the specified ID.
 * @param obj The array of JSON-LD node objects to search.
 * @param id The ID to search for.
 * @returns The index of the first element with the specified ID, or undefined if no such element is found.
 */
function jsonLdGetByID(obj, id) {
  for (var i = 0; i < obj.length; i++) {
    const element = obj[i];
    if (element["@id"] === id) {
      return i;
    }
  }
  return undefined;
}
function jsonLdGetByType(obj, type) {
  let result = [];
  for (var i = 0; i < obj.length; i++) {
    const element = obj[i];
    if (element["@type"] !== undefined && Array.isArray(element["@type"])) {
      for (let elementType of element["@type"]) {
        if (elementType === type || elementType.replace("http://", "https://") === type) {
          result.push(element["@id"]);
        }
      }
    }
    else if (element["@type"] === type) {
      result.push(element["@id"]);
    }
  }
  return result;
}
function jsonLdIsLocalID(obj, id) {
  return jsonLdGetByID(obj, id) !== undefined;
}
function jsonLdGetSubNodes(data, obj, path = "") {
  let subNodes = new Map();
  for (let key in obj) {
    if (!key.startsWith("@")) {
      let value = obj[key];
      if (Array.isArray(value)) {
        for (let element of value) {
          if (typeof element === "object") {
            //subNodes.push(...jsonLdGetSubNodes(data, element as jsonld.NodeObject, path + "/" + key));
            let subSubNodes = jsonLdGetSubNodes(data, element, path + "/" + key);
            for (let subSubNode of subSubNodes) {
              if (subNodes.has(subSubNode[0]) && Array.isArray(subNodes.get(subSubNode[0]))) {
                let pathObj = subNodes.get(subSubNode[0]);
                if (pathObj !== undefined) {
                  pathObj.push(...subSubNode[1]);
                }
              }
              else {
                subNodes.set(subSubNode[0], subSubNode[1]);
              }
            }
          }
        }
      }
    }
    else if (path != "" && key === "@id" && jsonLdIsLocalID(data, obj[key])) {
      if (subNodes.has(path) && Array.isArray(subNodes.get(path))) {
        let pathObj = subNodes.get(path);
        if (pathObj !== undefined) {
          pathObj.push(obj[key]);
        }
      }
      else {
        subNodes.set(path, [obj[key]]);
      }
    }
  }
  return subNodes;
}
function pathTraversal(obj, path) {
  let current = obj;
  for (let i = 0; i < path.length; i++) {
    if (current[path[i]] !== undefined) {
      current = current[path[i]];
    }
    else {
      return undefined;
    }
  }
  return current;
}
function pathTraversalSet(obj, path, value) {
  let previous = obj;
  let previousKey = "";
  let current = obj;
  for (let i = 0; i < path.length - 1; i++) {
    if (current[path[i]] !== undefined) {
      previous = current;
      previousKey = path[i];
      current = current[path[i]];
    }
    else {
      return;
    }
  }
  if (value === undefined) {
    delete current[path[path.length - 1]];
    if (Array.isArray(previous[previousKey]) && previous[previousKey].filter((x) => x !== undefined).length === 0) {
      delete previous[previousKey];
    }
  }
  else {
    current[path[path.length - 1]] = value;
  }
}
/**
 * Type guard that checks if the given object is null, undefined or an empty string.
 * @param obj - The object to check.
 * @returns A boolean indicating whether the object is nullish or not.
 */
function isNullish(obj) {
  return obj === undefined || obj === null || obj === "";
}

exports.isNullish = isNullish;
exports.jsonLdGetByID = jsonLdGetByID;
exports.jsonLdGetByType = jsonLdGetByType;
exports.jsonLdGetSubNodes = jsonLdGetSubNodes;
exports.jsonLdNodeToName = jsonLdNodeToName;
exports.jsonLdNodeToType = jsonLdNodeToType;
exports.jsonLdNodeToTypeIds = jsonLdNodeToTypeIds;
exports.pathTraversal = pathTraversal;
exports.pathTraversalSet = pathTraversalSet;

//# sourceMappingURL=utils-4d8ad9ee.js.map