import { h, proxyCustomElement, HTMLElement, createEvent, Host } from '@stencil/core/internal/client';
import { c as pathTraversal, a as jsonLdGetByID, p as pathTraversalSet } from './utils.js';
import { d as defineCustomElement$1 } from './inclde-view-link2.js';

class BasicEl {
  constructor(value, callback) {
    this.value = "";
    if (typeof value === 'string') {
      this.value = value;
    }
    else if (typeof value === 'number') {
      this.value = value;
    }
    else if (typeof value === 'boolean') {
      this.value = value;
    }
    else {
      console.error('Value type is not supported', value);
    }
    this.callback = callback;
  }
  handleChange(event) {
    if (event.target instanceof HTMLInputElement) {
      if (typeof this.value === 'string') {
        this.value = event.target.value;
      }
      else if (typeof this.value === 'number') {
        this.value = Number(event.target.value);
      }
      else if (typeof this.value === 'boolean') {
        this.value = event.target.checked;
      }
      if (this.callback !== undefined) {
        this.callback(this.value);
      }
    }
  }
  render(editable = true) {
    if (typeof this.value === 'string') {
      return (h("input", { type: "text", value: this.value, onInput: (event) => this.handleChange(event), readOnly: !editable, disabled: !editable }));
    }
    else if (typeof this.value === 'number') {
      return (h("input", { type: "number", value: this.value, onInput: (event) => this.handleChange(event), readOnly: !editable, disabled: !editable }));
    }
    else if (typeof this.value === 'boolean') {
      return (h("input", { type: "checkbox", checked: this.value, onInput: (event) => this.handleChange(event), readOnly: !editable, disabled: !editable }));
    }
    else {
      return (h("span", null, "Value type is not supported"));
    }
  }
}
BasicEl.id = '_direct';

class JsonFallbackEl {
  constructor(value, callback) {
    this.value = "";
    this.value = JSON.stringify(value);
    this.callback = callback;
  }
  handleChange(event) {
    if (event.target instanceof HTMLInputElement) {
      this.value = JSON.parse(event.target.value);
      if (this.callback !== undefined) {
        this.callback(this.value);
      }
    }
  }
  render() {
    return (h("input", { type: "text", value: this.value, onInput: (event) => this.handleChange(event) }));
  }
}
JsonFallbackEl.id = '_fallback';

class ExternalEl {
  //private callback?: (value: any) => void;
  constructor(value) {
    this.value = "";
    if (typeof value['@id'] === 'string') {
      this.value = value['@id'];
    }
    else {
      console.error('Value is not of type string', value);
    }
    //this.callback = callback;
  }
  render() {
    return (h("a", { href: this.value, target: '_blank' }, this.value));
  }
}
ExternalEl.id = '@id';

class ValueobjectEl {
  constructor(value, callback) {
    if (value["@value"] !== undefined) {
      let callbackWrapper = (newValue) => {
        value["@value"] = newValue;
        if (callback !== undefined) {
          callback(value);
        }
      };
      this.basicWrapper = new BasicEl(value["@value"], callbackWrapper);
    }
    else {
      console.error('Not a Value Object', value);
    }
  }
  render() {
    if (this.basicWrapper !== undefined) {
      return this.basicWrapper.render();
    }
    else {
      return (h("span", null, "Not a Value Object"));
    }
  }
}
ValueobjectEl.id = '@value';

class PropertyValueEl {
  constructor(value, callback) {
    this.name = '';
    let localValue;
    if (value["http://schema.org/value"] !== undefined && value["http://schema.org/value"].length == 1) {
      this.value = value;
      localValue = value["http://schema.org/value"][0]["@value"];
    }
    else {
      console.error("PropertyValue must have a value");
      return;
    }
    if (value["http://schema.org/name"] !== undefined && value["http://schema.org/name"].length == 1) {
      this.name = value["http://schema.org/name"][0]["@value"];
    }
    else {
      console.error("PropertyValue must have a name");
      return;
    }
    let callbackWrapper = (newValue) => {
      value["http://schema.org/value"][0]["@value"] = newValue;
      if (callback !== undefined) {
        callback(value);
      }
    };
    this.basicWrapper = new BasicEl(localValue, callbackWrapper);
  }
  render() {
    if (this.basicWrapper !== undefined) {
      return (h("span", { class: "property-container" },
        h("span", { class: "property-name" }, this.name + ":"),
        h("span", { class: "property-value" }, this.basicWrapper.render())));
    }
    else {
      return (h("span", null, "This Property Value is not supported"));
    }
  }
}
PropertyValueEl.id = 'http://schema.org/PropertyValue';

let customElementList = [ExternalEl, ValueobjectEl, PropertyValueEl];
function createCustomElement(id, value, callback) {
  if (id === "_direct") {
    return new BasicEl(value, callback);
  }
  for (let customElement of customElementList) {
    if (customElement.id.toLowerCase() === id.toLowerCase() || customElement.id.toLowerCase() === id.replace('http://', 'https://').toLowerCase()) {
      return new customElement(value, callback);
    }
  }
  return new JsonFallbackEl(value, callback);
}
function getAllCustomElements() {
  return customElementList.map((customElement) => customElement.id);
}

const incldeInputCss = ":host{display:block}.property-container{display:flex;flex-grow:1;justify-content:space-between;align-items:stretch}.property-name{padding-right:1em}.property-value{flex-grow:1}input{width:100%;box-sizing:border-box;}button.edit-button{display:inline-block;vertical-align:middle;background:none;border:none;padding:0px;padding-left:5px;margin:0px;width:15px;height:15px;cursor:pointer}button.edit-button svg{width:15px;height:15px;fill:#777}button.edit-button:hover svg{fill:#000}button.edit-button:active svg{fill:#555}";

const IncldeInput = /*@__PURE__*/ proxyCustomElement(class IncldeInput extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.__attachShadow();
    this.elementSelected = createEvent(this, "elementSelected", 7);
    this.isComplex = false;
    this.data = [];
    this.selector = [];
    this.isEditable = true;
    this.value = undefined;
  }
  componentWillLoad() {
    this.initCustomElement();
  }
  watchSelector(_newValue, _oldValue) {
    this.initCustomElement();
  }
  initCustomElement() {
    this.customElement = undefined;
    this.isComplex = false;
    let updateValue = (newValue) => {
      pathTraversalSet(this.data, this.selector, newValue);
    };
    this.value = pathTraversal(this.data, this.selector);
    if (typeof this.value === 'string' || typeof this.value === 'number' || typeof this.value === 'boolean') {
      this.customElement = createCustomElement("_direct", this.value, updateValue);
    }
    else if (this.value['@value'] !== undefined) {
      this.customElement = createCustomElement("@value", this.value, updateValue);
    }
    else if (this.value['@id'] !== undefined) {
      let id_pos = jsonLdGetByID(this.data, this.value['@id']);
      if (id_pos === undefined) {
        this.customElement = createCustomElement("@id", this.value, updateValue);
      }
      else if (pathTraversal(this.data, [id_pos])['@type'] !== undefined) {
        let resolvedObj = pathTraversal(this.data, [id_pos]);
        // Check if type can be represented as a custom element
        for (let type in resolvedObj['@type']) {
          if (getAllCustomElements().includes(resolvedObj['@type'][type])) {
            let updateObjectValue = (newValue) => {
              pathTraversalSet(this.data, [id_pos], newValue);
            };
            this.customElement = createCustomElement(resolvedObj['@type'][type], resolvedObj, updateObjectValue);
            this.isComplex = true;
            this.linkLocation = id_pos;
            break;
          }
        }
        if (this.customElement === undefined) {
          this.linkLocation = id_pos;
        }
      }
    }
    else {
      console.error('Unknown type of input', this.value);
    }
  }
  switchToDetailedView() {
    this.elementSelected.emit({
      selectIndex: this.linkLocation,
    });
  }
  render() {
    var _a;
    let custom_render = (_a = this.customElement) === null || _a === void 0 ? void 0 : _a.render(this.isEditable);
    if (custom_render) {
      return (h(Host, { class: "property-container" }, custom_render, this.isComplex &&
        h("button", { class: "edit-button", onClick: () => this.switchToDetailedView() }, h("svg", { xmlns: "http://www.w3.org/2000/svg", viewBox: "0 0 512 512" }, h("path", { d: "M362.7 19.3L314.3 67.7 444.3 197.7l48.4-48.4c25-25 25-65.5 0-90.5L453.3 19.3c-25-25-65.5-25-90.5 0zm-71 71L58.6 323.5c-10.4 10.4-18 23.3-22.2 37.4L1 481.2C-1.5 489.7 .8 498.8 7 505s15.3 8.5 23.7 6.1l120.3-35.4c14.1-4.2 27-11.8 37.4-22.2L421.7 220.3 291.7 90.3z" })))));
    }
    else if (this.linkLocation !== undefined) {
      return (h(Host, null, h("inclde-view-link", { data: this.data, selector: this.linkLocation })));
    }
    else {
      return (h(Host, null, h("span", null, "Unknown Render Type")));
    }
  }
  static get watchers() { return {
    "selector": ["watchSelector"]
  }; }
  static get style() { return incldeInputCss; }
}, [1, "inclde-input", {
    "data": [16],
    "selector": [16],
    "isEditable": [4, "is-editable"],
    "value": [32]
  }, undefined, {
    "selector": ["watchSelector"]
  }]);
function defineCustomElement() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["inclde-input", "inclde-view-link"];
  components.forEach(tagName => { switch (tagName) {
    case "inclde-input":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, IncldeInput);
      }
      break;
    case "inclde-view-link":
      if (!customElements.get(tagName)) {
        defineCustomElement$1();
      }
      break;
  } });
}

export { IncldeInput as I, defineCustomElement as d };

//# sourceMappingURL=inclde-input2.js.map